require 'sudoku/model/coordinate'

module Sudoku
  # Represents a two dimensional coordinates in the Sudoku grid.
  class CellCoordinates
    include Comparable
    attr_reader :coordinate_x, :coordinate_y

    # Will initialize this cell coordinate.
    # * +x+ horizontal coordinate.
    # * +y+ vertical coordinate.
    def initialize(x, y)
      @coordinate_x = Sudoku::Coordinate.new(x)
      @coordinate_y = Sudoku::Coordinate.new(y)
    end

    def ==(other)
      self.class == other.class &&
        @coordinate_x == other.coordinate_x &&
        @coordinate_y == other.coordinate_y
    end

    alias_method :eql?, :==

    def hash
      @coordinate_x.coordinate ^ @coordinate_y.coordinate # XOR, from the doc
    end

    def <=>(other)
      return @coordinate_y <=> other.coordinate_y if
        @coordinate_y != other.coordinate_y
      @coordinate_x <=> other.coordinate_x
    end

    # Will return this coordinate in the format [x, y]
    def to_s
      "[#{coordinate_x}, #{coordinate_y}]"
    end

    # Will return true, if this coordinate belongs to area given
    # by two ranges of coordinates.
    # * +x+ range of x coordinates to accept.
    # * +y+ range of y coordinates to accept.
    def in_area(x, y)
      return true if x.include?(@coordinate_x) && y.include?(@coordinate_y)
      false
    end

    # Will return randomly generated coordinate.
    def self.random
      CellCoordinates.new(Random.rand(0..8), Random.rand(0..8))
    end
  end
end
