module Sudoku
  # Represents a cell coordinate in one dimension of the Sudoku
  # grid.
  class Coordinate
    include Comparable

    attr_reader :coordinate

    # Will initialize this coordinate.
    # * +coordinate+ coordinate value, which must be in the 0-8 interval.
    def initialize(coordinate)
      fail ArgumentError,
           "Coordinate value must be 0-8, value #{coordinate} given." unless
            coordinate_valid?(coordinate)

      @coordinate = coordinate.to_i
    end

    # Will check if given number can be used as the valid coordinate,
    # which means, that it is a number in the interval
    # 0-8 inclusive. True is returned if the coordinate is valid,
    # false otherwise.
    # * +coordinate+ coordinate value to check
    def coordinate_valid?(coordinate)
      Coordinate.valid?(coordinate)
    end

    # Will return true, if the given coordinate is between 0-8
    # interval, inclusive.
    def self.valid?(coordinate)
      return true if coordinate >= 0 && coordinate < 9
      false
    end

    def to_s
      @coordinate.to_s
    end

    # Will convert this coordinate to integer.
    def to_i
      @coordinate
    end

    # Will convert this coordinate to integer.
    def to_int
      to_i
    end

    # Compares this cooridnate with the other one.
    def <=>(other)
      return @coordinate <=> other.coordinate if other.respond_to?(:coordinate)
      @coordinate <=> other.to_i
    end

    # Will check, if the given element represents the same
    # one-dimensional coordiante.
    def eql?(other)
      @coordinate == other.to_i
    end
  end
end
